aboutsummaryrefslogtreecommitdiff
path: root/lib/lacp.c
diff options
context:
space:
mode:
authorEthan Jackson <ethan@nicira.com>2011-04-18 12:22:12 -0700
committerEthan Jackson <ethan@nicira.com>2011-04-19 13:49:36 -0700
commitcdcf42c664eeb6b260693fc182c1782669f7d39e (patch)
treeda5797da43740167835464e4795b332ab68cf5bc /lib/lacp.c
parent808f401eb60cfc335251d641658d525c5266f27e (diff)
lacp: Implement custom timing mode.
With this patch, the LACP module may be manually configured to use an arbitrary transmission rate set in the database.
Diffstat (limited to 'lib/lacp.c')
-rw-r--r--lib/lacp.c64
1 files changed, 54 insertions, 10 deletions
diff --git a/lib/lacp.c b/lib/lacp.c
index 38ce595a..a7f66a24 100644
--- a/lib/lacp.c
+++ b/lib/lacp.c
@@ -48,7 +48,8 @@ struct lacp {
struct hmap slaves; /* Slaves this LACP object controls. */
struct slave *key_slave; /* Slave whose ID will be the aggregation key. */
- bool fast; /* Fast or Slow LACP time. */
+ enum lacp_time lacp_time; /* Fast, Slow or Custom LACP time. */
+ long long int custom_time; /* LACP_TIME_CUSTOM transmission rate. */
bool strict; /* True if in strict mode. */
bool negotiated; /* True if LACP negotiations were successful. */
bool update; /* True if lacp_update() needs to be called. */
@@ -188,7 +189,8 @@ lacp_configure(struct lacp *lacp, const struct lacp_settings *s)
}
lacp->active = s->active;
- lacp->fast = s->fast;
+ lacp->lacp_time = s->lacp_time;
+ lacp->custom_time = MAX(TIME_UPDATE_INTERVAL, s->custom_time);
}
/* Returns true if 'lacp' is configured in active mode, false if 'lacp' is
@@ -207,10 +209,23 @@ lacp_process_pdu(struct lacp *lacp, const void *slave_,
const struct lacp_pdu *pdu)
{
struct slave *slave = slave_lookup(lacp, slave_);
+ long long int tx_rate;
+
+ switch (lacp->lacp_time) {
+ case LACP_TIME_FAST:
+ tx_rate = LACP_FAST_TIME_TX;
+ break;
+ case LACP_TIME_SLOW:
+ tx_rate = LACP_SLOW_TIME_TX;
+ break;
+ case LACP_TIME_CUSTOM:
+ tx_rate = lacp->custom_time;
+ break;
+ default: NOT_REACHED();
+ }
slave->status = LACP_CURRENT;
- timer_set_duration(&slave->rx, LACP_RX_MULTIPLIER *
- (lacp->fast ? LACP_FAST_TIME_TX : LACP_SLOW_TIME_TX));
+ timer_set_duration(&slave->rx, LACP_RX_MULTIPLIER * tx_rate);
slave->ntt_actor = pdu->partner;
@@ -363,15 +378,21 @@ lacp_run(struct lacp *lacp, lacp_send_pdu *send_pdu)
if (timer_expired(&slave->tx)
|| !info_tx_equal(&actor, &slave->ntt_actor)) {
+ long long int duration;
slave->ntt_actor = actor;
compose_lacp_pdu(&actor, &slave->partner, &pdu);
send_pdu(slave->aux, &pdu);
- timer_set_duration(&slave->tx,
- (slave->partner.state & LACP_STATE_TIME
- ? LACP_FAST_TIME_TX
- : LACP_SLOW_TIME_TX));
+ if (lacp->lacp_time == LACP_TIME_CUSTOM) {
+ duration = lacp->custom_time;
+ } else {
+ duration = (slave->partner.state & LACP_STATE_TIME
+ ? LACP_FAST_TIME_TX
+ : LACP_SLOW_TIME_TX);
+ }
+
+ timer_set_duration(&slave->tx, duration);
}
}
}
@@ -487,10 +508,18 @@ slave_set_defaulted(struct slave *slave)
static void
slave_set_expired(struct slave *slave)
{
+ struct lacp *lacp = slave->lacp;
+
slave->status = LACP_EXPIRED;
slave->partner.state |= LACP_STATE_TIME;
slave->partner.state &= ~LACP_STATE_SYNC;
- timer_set_duration(&slave->rx, LACP_RX_MULTIPLIER * LACP_FAST_TIME_TX);
+
+ /* The spec says we should wait LACP_RX_MULTIPLIER * LACP_FAST_TIME_TX.
+ * This doesn't make sense when using custom times which can be much
+ * smaller than LACP_FAST_TIME. */
+ timer_set_duration(&slave->rx, (lacp->lacp_time == LACP_TIME_CUSTOM
+ ? lacp->custom_time
+ : LACP_RX_MULTIPLIER * LACP_FAST_TIME_TX));
}
static void
@@ -502,7 +531,7 @@ slave_get_actor(struct slave *slave, struct lacp_info *actor)
state |= LACP_STATE_ACT;
}
- if (slave->lacp->fast) {
+ if (slave->lacp->lacp_time != LACP_TIME_SLOW) {
state |= LACP_STATE_TIME;
}
@@ -691,6 +720,21 @@ lacp_unixctl_show(struct unixctl_conn *conn,
}
ds_put_cstr(&ds, "\n");
+ ds_put_cstr(&ds, "\tlacp_time: ");
+ switch (lacp->lacp_time) {
+ case LACP_TIME_FAST:
+ ds_put_cstr(&ds, "fast\n");
+ break;
+ case LACP_TIME_SLOW:
+ ds_put_cstr(&ds, "slow\n");
+ break;
+ case LACP_TIME_CUSTOM:
+ ds_put_format(&ds, "custom (%lld)\n", lacp->custom_time);
+ break;
+ default:
+ ds_put_cstr(&ds, "unknown\n");
+ }
+
HMAP_FOR_EACH (slave, node, &lacp->slaves) {
char *status;
struct lacp_info actor;